function x_dot = spmCT(t,u,x,P,Pk)
%% Continuous-time structure preserving model
% Inputs:
%   u       control input
%   x       state vector
%   P       controller parameters (see fmincon_wrapper.m)
%   Pk      load vector P(k)
% Output:
%   x_dot   state derivative

% Copyright (c) 2018, Andreas Reason Dahl
% All rights reserved. Published under MIT. See LICENSE.txt.
% Please include the reference given in reference.txt whenever using this.

%% Update network matrix, for genset disconnection
y1 = P.y_vec(1);
y2 = P.y_vec(2);
y3 = P.y_vec(3);
y4 = P.y_vec(4);

b = 1; % bus-tie

% Lines   1   2   3   4   5   6   7     | Nodes | Description
P.B =   [-1   0   0  -1   0   0   0 ;   % 1     | Load 1
          1  -b   0   0 -y2   0   0 ;   % 2     | Load 2
          0   b  -1   0   0 -y3   0 ;   % 3     | Load 3
          0   0   1   0   0   0 -y4 ;   % 4     | Load 4
          0   0   0  y1   0   0   0 ;   % 5     | Genset 1 (diesel)
          0   0   0   0  y2   0   0 ;   % 6     | Genset 2 (gas)
          0   0   0   0   0  y3   0 ;   % 7     | Genset 3 (diesel)
          0   0   0   0   0   0  y4 ;]; % 8     | Genset 4 (gas)

%% Calculate SPM

f_flow = P.B*(P.B_sus.*sin(-P.B'*x(1:8)));       % Power flow
f_gov  = u-(x(9:12)-1)/0.05;                     % Governor
f_COI  = (ones(1,4)*P.M*x(9:12))/sum(diag(P.M)); % Centre of inertia

x_dot  = [P.D\(P.fU*f_flow + Pk)
           (x(9:12)-f_COI)*P.omega_B
           P.M\(f_gov + sum(P.S)./P.S.*P.fL*f_flow./x(9:12))];
end